#ifndef __COMMON_H__
#define __COMMON_H__

/*
  QtCurve (C) Craig Drummond, 2003 Craig.Drummond@lycos.co.uk

----

  Released under the GNU General Public License (GPL) v2.

----

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files(the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/

/* This file is quite 'hacky' as it contains lots of static function definitions - but I want to share the code
   between Qt and Gtk, but not polute the namespace with exported functions... */

#include <string.h>
#include <math.h>
#include "config.h"

#ifdef __cplusplus
#include <qstring.h>
#include <qsettings.h>
#include <qconfig.h>
#ifdef QTC_CONFIG_DIALOG
#include <qapplication.h>
#endif
#else
#include <glib.h>
#endif

#ifdef __cplusplus
typedef QColor color;
#else
typedef gboolean bool;
typedef GdkColor color;
#define true TRUE
#define false FALSE
#endif

#define QTC_GROUP        "Settings"
#define QTC_DESCR_GROUP  "Description"

#define QTC_CHECK_SIZE   13
#define QTC_RADIO_SIZE   13
#define QTC_MIN_BTN_SIZE 8
#define QTC_LV_SIZE      7

#define NUM_STD_SHADES   6
#define NUM_EXTRA_SHADES 3

#define TOTAL_SHADES     NUM_STD_SHADES+NUM_EXTRA_SHADES
#define ORIGINAL_SHADE   TOTAL_SHADES

#define SHADE_ORIG_HIGHLIGHT NUM_STD_SHADES
#define SHADE_4_HIGHLIGHT    NUM_STD_SHADES+1
#define SHADE_2_HIGHLIGHT    NUM_STD_SHADES+2

/* 3d effect - i.e. buttons, etc */
#define QTC_SHADES \
    static const double shades[11][NUM_STD_SHADES]=\
    { \
      { 1.07, 1.03, 0.91, 0.780, 0.834, 0.82 }, \
      { 1.08, 1.03, 0.91, 0.781, 0.835, 0.80 }, \
      { 1.09, 1.03, 0.91, 0.782, 0.836, 0.78 }, \
      { 1.10, 1.04, 0.91, 0.783, 0.837, 0.76 }, \
      { 1.11, 1.04, 0.91, 0.784, 0.838, 0.74 }, \
      { 1.12, 1.05, 0.91, 0.785, 0.840, 0.72 }, \
      { 1.13, 1.05, 0.91, 0.786, 0.842, 0.70 }, \
      { 1.14, 1.06, 0.91, 0.787, 0.844, 0.68 }, /* default */ \
      { 1.16, 1.06, 0.91, 0.788, 0.846, 0.53 }, \
      { 1.18, 1.07, 0.91, 0.789, 0.848, 0.48 }, \
      { 1.20, 1.07, 0.91, 0.790, 0.850, 0.43 }  \
    };

#define QT_FRAME_DARK_SHADOW 2

#define QTC_SHADE(c, s) \
    (c>10 || c<0 || s>=NUM_STD_SHADES || s<0 ? 1.0 : shades[c][s])

#define TAB_APPEARANCE(A)   (A) /* (APPEARANCE_GLASS==(A) ? APPEARANCE_GRADIENT : (A)) */

#define QTC_ETCHED_DARK            0.95
#define SHADE_BEVEL_GRAD_LIGHT     1.05
#define SHADE_BEVEL_GRAD_DARK      0.93
#define SHADE_BEVEL_GRAD_SEL_LIGHT 1.05
#define SHADE_BEVEL_GRAD_SEL_DARK  0.95

#define SHADE_BEVEL_MENU_ITEM_LIGHT 1.07
#define SHADE_BEVEL_MENU_ITEM_DARK  0.85

#define SHADE_SLIDER_LIGHT         1.1
#define SHADE_SLIDER_DARK          0.8

#define SHADE_SBAR_LIGHT           1.02
#define SHADE_SBAR_DARK            0.95

#define SHADE_MENU_LIGHT           1.02
#define SHADE_MENU_DARK            0.96

#define SHADE_TAB_SEL_LIGHT        1.1
#define SHADE_TAB_SEL_DARK         1.0

#define SHADE_BOTTOM_TAB_SEL_LIGHT 1.0
#define SHADE_BOTTOM_TAB_SEL_DARK  0.96

#define SHADE_GLASS_TOP_A(A, W) (APPEARANCE_DULL_GLASS==A \
                                    ? (WIDGET_DEF_BUTTON==W ? 1.05 : 1.20) \
                                    : (WIDGET_DEF_BUTTON==W ? 1.08 : 1.7))
#define SHADE_GLASS_TOP_B(A, W) 0.96
#define SHADE_GLASS_BOT_A(A) 0.99
#define SHADE_GLASS_BOT_B(A) (APPEARANCE_DULL_GLASS==A ? 1.05 : 1.16)

#define IS_GLASS(A) (APPEARANCE_DULL_GLASS==A || APPEARANCE_SHINY_GLASS==A)
#define IS_FLAT(A)  (APPEARANCE_FLAT==A || APPEARANCE_RAISED==A)
#define SHADE_SELECTION_TOP 1.15
#define SHADE_SELECTION_BOT 0.9

#ifdef __cplusplus
#define MENUBAR_DARK_LIMIT 160
#define TOO_DARK(A) ((A).red()<MENUBAR_DARK_LIMIT || (A).green()<MENUBAR_DARK_LIMIT || (A).blue()<MENUBAR_DARK_LIMIT)
#else
#define MENUBAR_DARK_LIMIT (160<<8)
#define TOO_DARK(A) ((A).red<MENUBAR_DARK_LIMIT || (A).green<MENUBAR_DARK_LIMIT || (A).blue<MENUBAR_DARK_LIMIT)
#endif

#define DEFAULT_HIGHLIGHT_FACTOR   1.05
#define MAX_HIGHLIGHT_FACTOR         50
#define MIN_HIGHLIGHT_FACTOR        -50
#define MENUBAR_DARK_FACTOR        0.97
#define POPUPMENU_LIGHT_FACTOR     1.15
#define INACTIVE_HIGHLIGHT_FACTOR  1.15

#define USE_SHADED_MENU_BAR_COLORS (SHADE_CUSTOM==opts.shadeMenubars || SHADE_BLEND_SELECTED==opts.shadeMenubars)
#define MENUBAR_GLASS_SELECTED_DARK_FACTOR 0.9
#define GLASS_BORDER     0.4
#define BEVEL_BORDER(w)    (WIDGET_LISTVIEW_HEADER==w ? 6 : 4)
#define SHADE_BEVEL_TOP    1.07
#define SHADE_BEVEL_MID_TOP    1.03
#define SHADE_BEVEL_MID_BOT    0.975
#define SHADE_BEVEL_BOT(w) (WIDGET_LISTVIEW_HEADER==(w) ? 0.88 : 0.90)

#define NUM_SPLITTER_DASHES 21

#define WIDGET_BUTTON(w) (WIDGET_STD_BUTTON==w || WIDGET_DEF_BUTTON==w || WIDGET_TOGGLE_BUTTON==w || WIDGET_CHECKBOX==w)
#ifdef __cplusplus
#define ETCH_WIDGET(w) (WIDGET_STD_BUTTON==w || WIDGET_DEF_BUTTON==w || WIDGET_TOGGLE_BUTTON==w)
#else
#define ETCH_WIDGET(w) (WIDGET_STD_BUTTON==w || WIDGET_DEF_BUTTON==w || WIDGET_TOGGLE_BUTTON==w || \
                        WIDGET_SPIN_UP==w || WIDGET_SPIN_DOWN==w)
#endif
#define COLORED_BORDER_SIZE 3
#define PROGRESS_CHUNK_WIDTH 10
#define PROGRESS_ANIMATION 100
#define MIN_SLIDER_SIZE(A) (LINE_DOTS==A ? 24 : 20)

#define QTC_NORM_TAB_APP (APPEARANCE_BEVELLED==opts.appearance ? APPEARANCE_GRADIENT : opts.appearance)
#define QTC_SEL_TAB_APP (APPEARANCE_INVERTED==opts.appearance ? APPEARANCE_FLAT : (QTC_NORM_TAB_APP))

typedef enum
{
    PIX_RADIO_BORDER,
    PIX_RADIO_LIGHT,
    PIX_RADIO_ON,
    PIX_CHECK
} EPixmap;

typedef enum
{
    WIDGET_TAB_TOP,
    WIDGET_TAB_BOT,
    WIDGET_STD_BUTTON,
    WIDGET_DEF_BUTTON,
    WIDGET_LISTVIEW_HEADER,
    WIDGET_SLIDER,
    WIDGET_SLIDER_TROUGH,
    WIDGET_SB_SLIDER,
    WIDGET_SB_TROUGH,
    WIDGET_CHECKBOX,
    WIDGET_TOGGLE_BUTTON,
    WIDGET_MENU_ITEM,
    WIDGET_PROGRESSBAR,
#ifndef __cplusplus
    WIDGET_SPIN_UP,
    WIDGET_SPIN_DOWN,
#endif
    WIDGET_SPIN,
    WIDGET_OTHER
} EWidget;

#define IS_SLIDER(W)        (WIDGET_SLIDER==W || WIDGET_SB_SLIDER==W)
#define IS_TOGGLE_BUTTON(W) (WIDGET_TOGGLE_BUTTON==W || WIDGET_CHECKBOX==W)

#ifndef QTC_CONFIG_DIALOG
static double getWidgetShade(EWidget w, bool light, bool sunken)
{
    switch(w)
    {
        case WIDGET_MENU_ITEM:
            return light ? SHADE_BEVEL_MENU_ITEM_LIGHT : SHADE_BEVEL_MENU_ITEM_DARK;
        case WIDGET_SB_TROUGH:
            return light ? SHADE_SBAR_LIGHT : SHADE_SBAR_DARK;
        case WIDGET_SLIDER_TROUGH:
            return light ? SHADE_SLIDER_LIGHT : SHADE_SLIDER_DARK;
        default:
            return light
                    ? sunken
                        ? SHADE_BEVEL_GRAD_SEL_LIGHT
                        : SHADE_BEVEL_GRAD_LIGHT
                    : sunken
                        ? SHADE_BEVEL_GRAD_SEL_DARK
                        : SHADE_BEVEL_GRAD_DARK;
    }
}
#endif

typedef enum
{
    CORNER_TL = 0x1,
    CORNER_TR = 0x2,
    CORNER_BR = 0x4,
    CORNER_BL = 0x8
} ECornerBits;

#define ROUNDED_NONE            0x0
#define ROUNDED_TOP             (CORNER_TL|CORNER_TR)
#define ROUNDED_BOTTOM          (CORNER_BL|CORNER_BR)
#define ROUNDED_LEFT            (CORNER_TL|CORNER_BL)
#define ROUNDED_RIGHT           (CORNER_TR|CORNER_BR)
#define ROUNDED_TOPRIGHT        CORNER_TR
#define ROUNDED_BOTTOMRIGHT     CORNER_BR
#define ROUNDED_TOPLEFT         CORNER_TL
#define ROUNDED_BOTTOMLEFT      CORNER_BL
#define ROUNDED_ALL             (CORNER_TL|CORNER_TR|CORNER_BR|CORNER_BL)

typedef enum
{
    IND_CORNER,
    IND_FONT_COLOR,
    IND_COLORED,
    IND_NONE
} EDefBtnIndicator;

typedef enum
{
    LINE_SUNKEN,
    LINE_DOTS,
    LINE_DASHES
} ELine;

#define LINE_NONE LINE_DASHES

typedef enum
{
    TB_NONE,
    TB_LIGHT,
    TB_DARK
} ETBarBorder;

typedef enum
{
    BORDER_FLAT,
    BORDER_RAISED,
    BORDER_SUNKEN
} EBorder;

typedef enum
{
    APPEARANCE_FLAT,
    APPEARANCE_DULL_GLASS,
    APPEARANCE_SHINY_GLASS,
    APPEARANCE_GRADIENT,
    APPEARANCE_INVERTED,
    APPEARANCE_BEVELLED,
    APPEARANCE_RAISED
} EAppearance;

typedef enum
{
    SHADE_NONE,
    SHADE_BLEND_SELECTED,
    SHADE_SELECTED,
    SHADE_DARKEN = SHADE_SELECTED ,  /* For menubar only! */
    SHADE_CUSTOM
} EShade;

typedef enum
{
    SCROLLBAR_KDE,
    SCROLLBAR_WINDOWS,
    SCROLLBAR_PLATINUM,
    SCROLLBAR_NEXT,
    SCROLLBAR_NONE
} EScrollbar;

#define DEF_IND_STR                "fontcolor"
#define DEF_LINE_STR               "dots"
#define DEF_TB_BORDER              "none"
#define DEF_APPEARANCE_STR         "bevelled"
#define DEF_MENU_APPEARANCE_STR    "gradient"
#define DEF_TOOLBAR_APPEARANCE_STR "gradient"
#define DEF_SLIDER_SHADE_STR       "selected"
#define DEF_TBS_STR                "dots"
#define DEF_COLOR_STR              "background"
#define DEF_TOOLBAR_SHADE_STR      "none"

#ifndef QTC_COMMON_NO_FUNCTIONS

#ifndef QTC_COMMON_ONLY_COVERTERS
#define QTC_MIN(a, b) ((a) < (b) ? (a) : (b))
#define QTC_MAX(a, b) ((b) < (a) ? (a) : (b))

static bool equal(double d1, double d2)
{
    return (fabs(d1 - d2) < 0.0001);
}

static char checkBounds(int num)
{
    return num < 0   ? 0   :
           num > 255 ? 255 :
                       num;
}

static void adjustPix(unsigned char *data, int numChannels, int w, int h, int stride,
                      int ro, int go, int bo, double shade)
{
    int width=w*numChannels,
        offset=0,
        row,
        r=(int)((ro*shade)+0.5),
        g=(int)((go*shade)+0.5),
        b=(int)((bo*shade)+0.5);

    for(row=0; row<h; ++row)
    {
        int column;

        for(column=0; column<width; column+=numChannels)
        {
            int source=(int)data[offset+column+1];

#if defined  __cplusplus
#if Q_BYTE_ORDER == Q_BIG_ENDIAN
            /* ARGB */
            data[offset+column+1] = checkBounds(r-source);
            data[offset+column+2] = checkBounds(g-source);
            data[offset+column+3] = checkBounds(b-source);
#else
            /* BGRA */
            data[offset+column] = checkBounds(b-source);
            data[offset+column+1] = checkBounds(g-source);
            data[offset+column+2] = checkBounds(r-source);
#endif
#else
            /* GdkPixbuf is RGBA */
            data[offset+column] = checkBounds(r-source);
            data[offset+column+1] = checkBounds(g-source);
            data[offset+column+2] = checkBounds(b-source);
#endif

        }
        offset+=stride;
    }
}

#endif /* QTC_COMMON_ONLY_COVERTERS */

#endif /* QTC_COMMON_NO_FUNCTIONS */

#ifdef __cplusplus
struct Options
#else
typedef struct
#endif
{
    int              contrast;
    double           highlightFactor;
    bool             embolden,
                     lighterPopupMenuBgnd,
                     highlightTab,
                     animatedProgress,
                     fixParentlessDialogs,
                     customMenuTextColor,
                     coloredMouseOver,
                     menubarMouseOver,
                     shadeMenubarOnlyWhenActive,
                     thinnerMenuItems,
                     shadowButtons,
                     stdFocus;
    ETBarBorder      toolbarBorders;
    EDefBtnIndicator defBtnIndicator;
    ELine            sliderThumbs,
                     handles,
                     toolbarSeparators,
                     splitters;
    EAppearance      appearance,
                     menubarAppearance,
                     toolbarAppearance;
    EShade           shadeSliders,
                     shadeMenubars;
    EScrollbar       scrollbarType;
    color            customMenubarsColor,
                     customSlidersColor,
                     customMenuNormTextColor,
                     customMenuSelTextColor;
#ifndef __cplusplus
} Options;
#else
};
#endif


#endif

